@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2006 Google Inc.
+# Copyright 2005-2008 Google Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,8 +14,8 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 #
 """Dectalk voice definitions using ACSS.
 
@@ -25,11 +25,11 @@ Dectalk voice parameter settings.
 
 """
 
-__id__ = "$Id: dectalk.py,v 1.9 2006/11/09 18:43:48 wwalker Exp $"
+__id__ = "$Id: dectalk.py 3882 2008-05-07 18:22:10Z richb $"
 __author__ = "T. V. Raman"
-__version__ = "$Revision: 1.9 $"
-__date__ = "$Date: 2006/11/09 18:43:48 $"
-__copyright__ = "Copyright (c) 2005 Google Inc."
+__version__ = "$Revision: 3882 $"
+__date__ = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__copyright__ = "Copyright (c) 2005-2008 Google Inc."
 __license__ = "LGPL"
 
 
@@ -37,7 +37,7 @@ _defined_voices = {}
 
 # Map from ACSS dimensions to Dectalk settings:
 
-_table ={}
+_table = {}
 #family codes:
 
 _table['family'] = {
@@ -68,7 +68,7 @@ _table['family'] = {
 
 def _update_map(table, key, format,  settings):
     """Internal function to update acss->synth mapping."""
-    table[key] ={}
+    table[key] = {}
     for setting  in  settings:
         _table[key][setting[0]] = format % setting[1:]
 
@@ -104,9 +104,9 @@ _man_ap = [
     (9, 140, 95)
     ]
 
-_update_map(_table,('man', 'average-pitch'),
+_update_map(_table, ('man', 'average-pitch'),
             " ap %s hs %s ",_man_ap)
-_update_map(_table,('harry', 'average-pitch'),
+_update_map(_table, ('harry', 'average-pitch'),
             " ap %s hs %s ",_man_ap)
 
 _female_ap = [
@@ -338,7 +338,7 @@ _update_map(_table, ('child', 'pitch-range'),
 # and the quickness --a parameter that controls whether the final
 # frequency targets are completely achieved in the phonetic transitions.
 
-_male_stress =[
+_male_stress = [
     (0, 0, 0, 0, 0),
     (1, 3, 6, 20, 3),
     (2, 6, 12, 40, 6),
@@ -494,33 +494,40 @@ _update_map(_table, ('child', 'richness'),
             " ri %s sm %s ", _female_richness)
 
 
-def getrate(r):    return int(180 + 4*r)
+def getrate(r):
+    return int(180 + 4*r)
 
-def getvolume(v):  return int(10*v)
+def getvolume(v):
+    return int(10*v)
 
-def getvoicelist(): return _table['family'].keys()
+def getvoicelist(): 
+    return _table['family'].keys()
 
 def getvoice(acss):
     """Memoized function that returns  synthesizer code for
     specified  ACSS setting.
     Synthesizer code is a tupple of the form (open,close)
     where open sets the voice, and close resets it."""
-    name=acss.name()
-    if name in _defined_voices: return _defined_voices[name]
-    _defined_voices[name] =acss2voice(acss)
+    name = acss.name()
+    if name in _defined_voices:
+        return _defined_voices[name]
+    _defined_voices[name] = acss2voice(acss)
     return _defined_voices[name]
 
 def acss2voice(acss):
     """Return synthesizer code."""
     code = ""
-    familyName ='male'
+    familyName = 'male'
     if 'family' in acss:
         familyName = acss['family']['name']
         if familyName in _table['family']:
             code += _table['family'][familyName]
-    if 'rate' in acss: code += " :ra %s" % getrate(acss['rate'])
-    if 'punctuations' in acss: code += " :punc %s" %acss['punctuations']
-    if 'gain' in acss: code += " :volume set %s" % getvolume(acss['gain'])
+    if 'rate' in acss:
+        code += " :ra %s" % getrate(acss['rate'])
+    if 'punctuations' in acss:
+        code += " :punc %s" % acss['punctuations']
+    if 'gain' in acss:
+        code += " :volume set %s" % getvolume(acss['gain'])
     voice = ""
     dv = ""
     for d in ['average-pitch', 'pitch-range',
@@ -528,6 +535,8 @@ def acss2voice(acss):
         if d in acss:
             if _table.has_key((familyName, d)):
                 voice += _table[(familyName, d)][int(acss[d])]
-    if voice: dv = " :dv %s" % voice
-    if code or voice: code = "[%s  %s]" % (code, dv)
+    if voice:
+        dv = " :dv %s" % voice
+    if code or voice:
+        code = "[%s  %s]" % (code, dv)
     return (code, " [:np] ")
