@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2007 Sun Microsystems Inc.
+# Copyright 2004-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,15 +14,15 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Provides the FocusTrackingPresenter for Orca."""
 
-__id__        = "$Id: focus_tracking_presenter.py 2821 2007-09-12 14:28:07Z wwalker $"
-__version__   = "$Revision: 2821 $"
-__date__      = "$Date: 2007-09-12 10:28:07 -0400 (Wed, 12 Sep 2007) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__  = "$Id: focus_tracking_presenter.py 3882 2008-05-07 18:22:10Z richb $"
+__version__   = "$Revision: 3882 $"
+__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import gobject
@@ -30,21 +30,18 @@ import Queue
 import threading
 import time
 
-import atspi
+import pyatspi
 import braille
 import default
 import debug
 import input_event
 import orca_state
 import presentation_manager
-import rolenames
 import settings
 import speech
 
 from orca_i18n import _ # for gettext support
 
-import CORBA
-
 class FocusTrackingPresenter(presentation_manager.PresentationManager):
     """Maintain a set of scripts for all running applications, and
     also keeps the notion of an activeScript.  All object events are
@@ -55,6 +52,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
     def __init__(self):
 
+        presentation_manager.PresentationManager.__init__(self)
+
         # Dictionary that keeps count of event listeners by type.
         # This is a bit convoluted for now, but what happens is that
         # scripts tell the FocusTrackingPresenter to listen for
@@ -65,7 +64,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # is what delves them out, we keep at most one listener to avoid
         # receiving the same event twice in a row.
         #
-        self.registry        = atspi.Registry()
+        self.registry        = pyatspi.Registry
         self._knownScripts   = {}
         self._knownAppSettings = {}
         self._oldAppSettings = None
@@ -76,6 +75,16 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         self.setActiveScript(None)
 
+        # Initialize variable to make pylint happy.
+        #
+        self._defaultScript = None
+        self._appStateInfo = None
+        self._listenerCounts = None
+
+        if settings.listenAllEvents:
+            # Just listen to everything, always.
+            self._registerAllEvents()
+
         if settings.debugEventQueue:
             self._enqueueEventCount = 0
             self._dequeueEventCount = 0
@@ -113,6 +122,22 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                                                   eventType)
             del self._listenerCounts[eventType]
 
+    def _registerAllEvents(self):
+        """Register all top-level event types except for "mouse".
+        """
+        topLevelEvents = filter(lambda et: ':' not in et and 'mouse' not in et,
+                                pyatspi.EVENT_TREE)
+        for event_type in topLevelEvents:
+            self.registry.registerEventListener(
+                self._enqueueEvent, event_type)
+
+    def _deregisterAllEvents(self):
+        topLevelEvents = filter(lambda et: ':' not in et and 'mouse' not in et,
+                                pyatspi.EVENT_TREE)
+        for event_type in topLevelEvents:
+            self.registry.deregisterEventListener(
+                self._enqueueEvent, event_type)
+
     def _registerEventListeners(self, script):
         """Tells the FocusTrackingPresenter to listen for all
         the event types of interest to the script.
@@ -121,6 +146,10 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         - script: the script.
         """
 
+        if settings.listenAllEvents: 
+            # We are always listening to everything.
+            return
+
         for eventType in script.listeners.keys():
             self._registerEventListener(eventType)
 
@@ -131,6 +160,9 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         Arguments:
         - script: the script.
         """
+        if settings.listenAllEvents: 
+            # We are always listening to everything.
+            return
 
         for eventType in script.listeners.keys():
             self._deregisterEventListener(eventType)
@@ -207,38 +239,39 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # default script. Note that this search is restricted to the "orca"
         # package for now.
         #
-        if (not script) \
-            and app \
-            and app.__dict__.has_key("toolkitName") \
-            and app.toolkitName:
-
-            try:
-                debug.println(
-                    debug.LEVEL_FINE,
-                    "Looking for toolkit script %s.py..." % app.toolkitName)
-                module = __import__(app.toolkitName,
-                                    globals(),
-                                    locals(),
-                                    [''])
-                script = module.Script(app)
-                debug.println(debug.LEVEL_FINE,
-                              "...found %s.py" % name)
-            except ImportError:
-                debug.println(
-                    debug.LEVEL_FINE,
-                    "...could not find %s.py" % app.toolkitName)
-            except:
-                debug.printException(debug.LEVEL_SEVERE)
-                debug.println(
-                    debug.LEVEL_SEVERE,
-                    "While attempting to import %s" % app.toolkitName)
+        if (not script) and app and getattr(app, "toolkitName", None):
+            for package in scriptPackages:
+                if package:
+                    name = '.'.join((package, app.toolkitName))
+                else:
+                    name = app.toolkitName
+                try:
+                    debug.println(
+                        debug.LEVEL_FINE,
+                        "Looking for toolkit script %s.py..." % app.toolkitName)
+                    module = __import__(name,
+                                        globals(),
+                                        locals(),
+                                        [''])
+                    script = module.Script(app)
+                    debug.println(debug.LEVEL_FINE,
+                                  "...found %s.py" % name)
+                except ImportError:
+                    debug.println(
+                        debug.LEVEL_FINE,
+                        "...could not find %s.py" % app.toolkitName)
+                except:
+                    debug.printException(debug.LEVEL_SEVERE)
+                    debug.println(
+                        debug.LEVEL_SEVERE,
+                        "While attempting to import %s" % app.toolkitName)
 
         if not script:
             script = default.Script(app)
 
         return script
 
-    def _getScript(self, app):
+    def getScript(self, app):
         """Get a script for an app (and make it if necessary).  This is used
         instead of a simple calls to Script's constructor.
 
@@ -299,19 +332,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         """
 
         objectsRemoved = 0
-        for obj in atspi.Accessible._cache.values():
-            try:
-                if obj.state.count(atspi.Accessibility.STATE_DEFUNCT):
-                    atspi.Accessible.deleteAccessible(obj)
-                    objectsRemoved += 1
-                else:
-                    # Try to force a COMM_FAILURE
-                    #
-                    obj.toString()
-            except CORBA.COMM_FAILURE:
-                atspi.Accessible.deleteAccessible(obj)
-                objectsRemoved += 1
-
         debug.println(debug.LEVEL_FINEST,
                       "_cleanupCache: %d objects removed." % objectsRemoved)
 
@@ -321,7 +341,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         gc.collect()
         for obj in gc.garbage:
             try:
-                if isinstance(obj, atspi.Accessible):
+                if isinstance(obj, pyatspi.Accessibility.Accessible):
                     gc.garbage.remove(obj)
                     obj.__del__()
             except:
@@ -340,19 +360,10 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # didn't reclaim this time.
         #
         try:
-            apps = []
-            desktop = self.registry.desktop
-            for i in range(0, desktop.childCount):
-                try:
-                    acc = desktop.getChildAtIndex(i)
-                    app = atspi.Accessible.makeAccessible(acc)
-                    if app:
-                        apps.insert(0, app)
-                except:
-                    pass
+            desktop = self.registry.getDesktop(0)
 
             for app in self._knownScripts.keys():
-                if apps.count(app) == 0:
+                if app not in desktop:
                     script = self._knownScripts[app]
                     self._deregisterEventListeners(script)
 
@@ -362,7 +373,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     # cycle of the application referring to itself.
                     #
                     del self._knownScripts[app]
-                    app.app = None
                     del app
                     del script
         except:
@@ -384,7 +394,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         app = script.app
         settingsPackages = settings.settingsPackages
         moduleName = settings.getScriptModuleName(app)
-        module = None
 
         if moduleName and len(moduleName):
             for package in settingsPackages:
@@ -482,14 +491,15 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # this problem.]]]
         #
         try:
-            if event.source.role == rolenames.ROLE_TOOL_TIP:
+            if event.source.getRole() == pyatspi.ROLE_TOOL_TIP:
                 # Check that it's okay to present tool tips. Always present
                 # tooltips initiated by the user pressing Control-F1 on the
                 # keyboard.
                 #
                 if isinstance(orca_state.lastInputEvent, \
                               input_event.KeyboardEvent):
-                    if not orca_state.lastNonModifierKeyEvent.event_string == "F1":
+                    if not orca_state.lastNonModifierKeyEvent.event_string \
+                                                                     == "F1":
                         return
 
                     # Mouse move events don't update orca_state.lastInputEvent
@@ -513,8 +523,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         # either.
         #
         if event.type.startswith("object:children-changed:remove"):
-            if event.source == atspi.Accessible.makeAccessible(
-                                   self.registry.desktop):
+            if event.source == self.registry.getDesktop(0):
                 self._reclaimScripts()
                 self._cleanupCache()
                 if settings.debugMemoryUsage:
@@ -525,12 +534,12 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # We don't want to touch a defunct object.  It's useless and it
             # can cause hangs.
             #
-            if event.source \
-               and event.source.state.count(atspi.Accessibility.STATE_DEFUNCT):
-                debug.println(debug.LEVEL_FINEST,
-                              "IGNORING DEFUNCT OBJECT")
-                atspi.Accessible.deleteAccessible(event.source)
-                return
+            if event.source:
+                state = event.source.getState()
+                if state.contains(pyatspi.STATE_DEFUNCT):
+                    debug.println(debug.LEVEL_FINEST,
+                                  "IGNORING DEFUNCT OBJECT")
+                    return
 
             if (not debug.eventDebugFilter) \
                 or (debug.eventDebugFilter \
@@ -538,17 +547,11 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                 if not event.type.startswith("mouse:"):
                     debug.printDetails(debug.LEVEL_FINEST, "    ", event.source)
 
-        except CORBA.COMM_FAILURE:
-            debug.printException(debug.LEVEL_WARNING)
-            debug.println(debug.LEVEL_FINEST,
-                          "COMM_FAILURE above while processing event: " \
-                          + event.type)
-        except CORBA.OBJECT_NOT_EXIST:
+        except LookupError:
             debug.printException(debug.LEVEL_WARNING)
             debug.println(debug.LEVEL_WARNING,
-                          "OBJECT_NOT_EXIST above while processing event: " \
-                          + event.type)
-            atspi.Accessible.deleteAccessible(event.source)
+                          "LookupError above while processing event: %s" %\
+                          event.type)
             return
         except:
             debug.printException(debug.LEVEL_WARNING)
@@ -572,12 +575,13 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         oldLocusOfFocus = orca_state.locusOfFocus
         try:
             # If we've received a mouse event, then don't try to get
-            # event.source.app because the top most parent is of role
-            # unknown, which will cause an ERROR message to be displayed.
-            # See Orca bug #409731 for more details.
+            # event.source.getApplication() because the top most parent
+            # is of role unknown, which will cause an ERROR message to be
+            # displayed. See Orca bug #409731 for more details.
             #
             if not event.type.startswith("mouse:"):
-                s = self._getScript(event.source.app)
+                s = self.getScript(event.host_application or \
+                                      event.source.getApplication())
             else:
                 s = orca_state.activeScript
             if not s:
@@ -590,8 +594,8 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         while s and retryCount <= s.commFailureAttemptLimit:
             try:
-                if not event.source.state.count( \
-                                      atspi.Accessibility.STATE_ICONIFIED):
+                state = event.source.getState()
+                if not state.contains(pyatspi.STATE_ICONIFIED):
 
                     # [[[TODO: WDW - HACK we look for frame that get
                     # focus: as a means to detect active scripts
@@ -601,39 +605,38 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     # in use, and then issues a focus: event on the
                     # main window, which is a frame.]]]
                     #
-                    if event.type.startswith("window:activate") \
-                       or (event.type.startswith("focus:") \
-                           and (event.source.role == rolenames.ROLE_FRAME)):
-
+                    if (event.type == "window:activate") \
+                       or ((event.type.startswith("focus")) 
+                           and (event.source.getRole() == pyatspi.ROLE_FRAME)):
                         # We'll let someone else decide if it's important
                         # to stop speech or not.
                         #speech.stop()
                         debug.println(debug.LEVEL_FINE, "ACTIVE SCRIPT: " \
                                       + orca_state.activeScript.name)
 
-                        self.setActiveScript(self._getScript(event.source.app))
+                        self.setActiveScript(
+                            self.getScript(event.host_application or \
+                                              event.source.getApplication()))
 
                         # Load in the application specific settings for the
                         # app for this event (if found).
                         #
-                        appSettings = self.loadAppSettings( \
-                                                      orca_state.activeScript)
+                        self.loadAppSettings(orca_state.activeScript)
 
                         # Tell BrlTTY which commands we care about.
                         #
                         braille.setupKeyRanges(\
                             orca_state.activeScript.brailleBindings.keys())
-
                     s.processObjectEvent(event)
                     if retryCount:
                         debug.println(debug.LEVEL_WARNING,
                                       "  SUCCEEDED AFTER %d TRIES" % retryCount)
                 break
-            except CORBA.COMM_FAILURE:
+            except LookupError:
                 debug.printException(debug.LEVEL_WARNING)
                 debug.println(debug.LEVEL_WARNING,
-                              "COMM_FAILURE above while processing: " \
-                              + event.type)
+                              "LookupError above while processing: %s" % \
+                              event.type)
                 retryCount += 1
                 if retryCount <= s.commFailureAttemptLimit:
                     # We want the old locus of focus to be reset so
@@ -649,7 +652,6 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     debug.println(debug.LEVEL_WARNING,
                                   "  GIVING UP AFTER %d TRIES" \
                                   % (retryCount - 1))
-                    atspi.Accessible.deleteAccessible(event.source)
             except:
                 debug.printException(debug.LEVEL_WARNING)
                 break
@@ -671,11 +673,11 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         event = None
         if isinstance(e, input_event.KeyboardEvent):
-            if e.type == atspi.Accessibility.KEY_PRESSED_EVENT:
+            if e.type == pyatspi.KEY_PRESSED_EVENT:
                 debug.println(debug.LEVEL_ALL,
                               "----------> QUEUEING KEYPRESS '%s' (%d)"
                               % (e.event_string, e.hw_code))
-            elif e.type == atspi.Accessibility.KEY_RELEASED_EVENT:
+            elif e.type == pyatspi.KEY_RELEASED_EVENT:
                 debug.println(debug.LEVEL_ALL,
                               "----------> QUEUEING KEYRELEASE '%s' (%d)"
                               % (e.event_string, e.hw_code))
@@ -685,8 +687,11 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                           "----------> QUEUEING BRAILLE COMMAND %d" % e.event)
             event = e
         else:
-            # We ignore defunct objects and let the atspi module take
-            # care of them for us.
+            if e.type in settings.ignoredEventsList:
+                if settings.debugEventQueue:
+                    self._enqueueEventCount -= 1
+                return
+            # We ignore defunct objects.
             #
             if e.type.startswith("object:state-changed:defunct"):
                 if settings.debugEventQueue:
@@ -696,8 +701,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # We also generally do not like
             # object:property-change:accessible-parent events because
             # they indicate something is now whacked with the
-            # hierarchy, so we just ignore them and let the atspi
-            # module take care of it for us.
+            # hierarchy.
             #
             if e.type.startswith("object:property-change:accessible-parent"):
                 if settings.debugEventQueue:
@@ -708,20 +712,17 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # removed from the desktop.
             #
             if e.type.startswith("object:children-changed:remove") \
-                and (e.source != self.registry.desktop):
+                and (e.source != self.registry.getDesktop(0)):
                 if settings.debugEventQueue:
                     self._enqueueEventCount -= 1
                 return
 
-            # We create the event here because it will ref everything
-            # we want it to ref, thus allowing things to survive until
-            # they are processed on the gidle thread.
-            #
             # If the event doesn't have a source or that source is not marked
             # valid, then we don't care about this event. Just return.
             #
-            event = atspi.Event(e)
-            if not event.source or not event.source.valid:
+            
+            event = e
+            if not event.source:
                 debug.println(debug.LEVEL_FINEST,
                       "---------> IGNORING INVALID EVENT %s" % e.type)
                 if settings.debugEventQueue:
@@ -797,12 +798,12 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             event = self._eventQueue.get_nowait()
 
             if isinstance(event, input_event.KeyboardEvent):
-                if event.type == atspi.Accessibility.KEY_PRESSED_EVENT:
+                if event.type == pyatspi.KEY_PRESSED_EVENT:
                     debug.println(debug.LEVEL_ALL,
                                   "DEQUEUED KEYPRESS '%s' (%d) <----------" \
                                   % (event.event_string, event.hw_code))
                     pressRelease = "PRESS"
-                elif event.type == atspi.Accessibility.KEY_RELEASED_EVENT:
+                elif event.type == pyatspi.KEY_RELEASED_EVENT:
                     debug.println(debug.LEVEL_ALL,
                                   "DEQUEUED KEYRELEASE '%s' (%d) <----------" \
                                   % (event.event_string, event.hw_code))
@@ -877,6 +878,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             debug.println(debug.LEVEL_SEVERE,
                           "focus_tracking_presenter:_dequeueEvent: " \
                           + " the event queue is empty!")
+            rerun = False # destroy and don't call again
         except:
             debug.printException(debug.LEVEL_SEVERE)
 
@@ -964,7 +966,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         """Restores script and application state information."""
         try:
             for [app, appState] in self._appStateInfo:
-                script = self._getScript(app)
+                script = self.getScript(app)
                 script.setAppState(appState)
         except:
             pass
@@ -982,7 +984,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         self._restoreAppStates()
 
-        self.setActiveScript(self._getScript(None))
+        self.setActiveScript(self.getScript(None))
 
         # Tell BrlTTY which commands we care about.
         #
@@ -997,12 +999,23 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # Generate a fake window activation event so the application
             # can tell the user about itself.
             #
-            e = atspi.Event()
-            e.source   = win._acc
-            e.type     = "window:activate"
-            e.detail1  = 0
-            e.detail2  = 0
-            e.any_data = None
+            class _FakeEvent:
+                def __init__(self, source, eventType,
+                             detail1, detail2, any_data):
+                    self.source = source
+                    self.type = eventType
+                    self.detail1 = detail1
+                    self.detail2 = detail2
+                    self.any_data = any_data
+
+            class _FakeData:
+                def __init__(self):
+                    pass
+                def value(self):
+                    return None
+
+            fe = _FakeEvent(win, "window:activate", 0, 0, _FakeData())
+            e = pyatspi.event.Event(fe)
             self._enqueueEvent(e)
 
     def deactivate(self):
