@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2006 Sun Microsystems Inc.
+# Copyright 2005-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,8 +14,8 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Provides support for handling input events.  This provides several classes
 to define input events (InputEvent, KeyboardEvent, BrailleEvent,
@@ -23,10 +23,10 @@ MouseButtonEvent, MouseMotionEvent, and SpeechEvent), and also provides a
 InputEventHandler class.  It is intended that instances of InputEventHandler
 will be used which should be used to handle all input events."""
 
-__id__        = "$Id: input_event.py 3561 2008-02-11 18:11:45Z wwalker $"
-__version__   = "$Revision: 3561 $"
-__date__      = "$Date: 2008-02-11 13:11:45 -0500 (Mon, 11 Feb 2008) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: input_event.py 3882 2008-05-07 18:22:10Z richb $"
+__version__   = "$Revision: 3882 $"
+__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import pyatspi
@@ -76,12 +76,14 @@ class KeyboardEvent(InputEvent):
             if value < 32:
                 event_string = chr(value + 0x40)
 
-        # Eliminate modifiers we're not interested in.
+        # Filter out the NUMLOCK modifier -- it always causes problems.
         #
         mask = (1 << settings.MODIFIER_ORCA |
-                1 << pyatspi.MODIFIER_ALT |
                 1 << pyatspi.MODIFIER_SHIFT |
+                1 << pyatspi.MODIFIER_SHIFTLOCK |
                 1 << pyatspi.MODIFIER_CONTROL |
+                1 << pyatspi.MODIFIER_ALT |
+                1 << pyatspi.MODIFIER_META |
                 1 << pyatspi.MODIFIER_META2 |
                 1 << pyatspi.MODIFIER_META3)
         event.modifiers = event.modifiers & mask
