@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2007 Sun Microsystems Inc.
+# Copyright 2005-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,16 +14,16 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Manages the magnifier for orca.  [[[TODO: WDW - this is very very
 early in development.  One might even say it is pre-prototype.]]]"""
 
-__id__        = "$Id: mag.py 3387 2007-12-17 16:34:14Z joanied $"
-__version__   = "$Revision: 3387 $"
-__date__      = "$Date: 2007-12-17 11:34:14 -0500 (Mon, 17 Dec 2007) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__        = "$Id: mag.py 3882 2008-05-07 18:22:10Z richb $"
+__version__   = "$Revision: 3882 $"
+__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import bonobo
@@ -505,6 +505,7 @@ def __setupMagnifier(position, left=None, top=None, right=None, bottom=None,
         prefTop    = top or settings.magZoomerTop
         prefRight  = right or settings.magZoomerRight
         prefBottom = bottom or settings.magZoomerBottom
+    orca_state.zoomerType = position
     updateTarget = True
 
     # If we're not using composite, bad things will happen if we allow the
@@ -590,13 +591,16 @@ def __setupMagnifier(position, left=None, top=None, right=None, bottom=None,
     value = restore.get('magCrossHairColor', settings.magCrossHairColor)
     setMagnifierObjectColor("crosswire-color", value, False)
 
-    value = restore.get('enableMagCrossHair', settings.enableMagCrossHair)
-    setMagnifierCrossHair(value, False)
+    enableCrossHair = restore.get('enableMagCrossHair',
+                                  settings.enableMagCrossHair)
+    setMagnifierCrossHair(enableCrossHair, False)
 
     value = restore.get('enableMagCrossHairClip',
                         settings.enableMagCrossHairClip)
     setMagnifierCrossHairClip(value, False)
 
+    orca_state.mouseEnhancementsEnabled = enableCursor or enableCrossHair
+
 def __setupZoomer(restore=None):
     """Creates a zoomer in the magnifier
     Arguments:
@@ -843,10 +847,6 @@ def applySettings():
     _pointerFollowsZoomer = settings.magPointerFollowsZoomer
     _pointerFollowsFocus = settings.magPointerFollowsFocus
 
-    orca_state.zoomerType = settings.magZoomerType
-    orca_state.mouseEnhancementsEnabled = settings.enableMagCursor \
-                                          or settings.enableMagCrossHair
-
     #print "MAGNIFIER PROPERTIES:", _magnifier
     #__dumpPropertyBag(_magnifier)
     #print "ZOOMER PROPERTIES:", _zoomer
@@ -885,9 +885,13 @@ def magnifyAccessible(event, obj, extents=None):
         try:
             text = obj.queryText()
             if text and (text.caretOffset >= 0):
+                offset = text.caretOffset
+                if offset == text.characterCount:
+                    offset -= 1
                 [x, y, width, height] = \
-                    text.getCharacterExtents(text.caretOffset, 0)
-                haveSomethingToMagnify = True
+                    text.getCharacterExtents(offset, 0)
+                haveSomethingToMagnify = (width + height > 0)
+                    
         except:
             haveSomethingToMagnify = False
 
@@ -1176,7 +1180,35 @@ def setZoomerObjectSize(magProperty, size, updateScreen=True):
     if not _initialized:
         return
 
-    bonobo.pbclient_set_long(_zoomerPBag, magProperty, size)
+    if magProperty == "border-size":
+        try:
+            left = right = top = bottom = 0
+            if orca_state.zoomerType == settings.MAG_ZOOMER_TYPE_RIGHT_HALF:
+                left = size
+            elif orca_state.zoomerType == settings.MAG_ZOOMER_TYPE_LEFT_HALF:
+                right = size
+            elif orca_state.zoomerType == settings.MAG_ZOOMER_TYPE_TOP_HALF:
+                bottom = size
+            elif orca_state.zoomerType == settings.MAG_ZOOMER_TYPE_BOTTOM_HALF:
+                top = size
+            elif orca_state.zoomerType == settings.MAG_ZOOMER_TYPE_CUSTOM:
+                if _targetDisplayBounds.x1 > _sourceDisplayBounds.x1:
+                    left = size
+                if _targetDisplayBounds.x2 < _sourceDisplayBounds.x2:
+                    right = size
+                if _targetDisplayBounds.y1 > _sourceDisplayBounds.y1:
+                    top = size
+                if _targetDisplayBounds.y2 < _sourceDisplayBounds.y2:
+                    bottom = size
+                
+            bonobo.pbclient_set_long(_zoomerPBag, "border-size-left", left)
+            bonobo.pbclient_set_long(_zoomerPBag, "border-size-top", top)
+            bonobo.pbclient_set_long(_zoomerPBag, "border-size-right", right)
+            bonobo.pbclient_set_long(_zoomerPBag, "border-size-bottom", bottom)
+        except:
+            bonobo.pbclient_set_long(_zoomerPBag, "border-size", size)
+    else:
+        bonobo.pbclient_set_long(_zoomerPBag, magProperty, size)
 
     if updateScreen:
         _zoomer.markDirty(_roi)
@@ -1260,8 +1292,6 @@ def setZoomerMagFactor(x, y, updateScreen=True):
 
     global _minROIX
     global _minROIY
-    global _maxROIX
-    global _maxROIY
 
     if not _initialized:
         return
@@ -1270,20 +1300,15 @@ def setZoomerMagFactor(x, y, updateScreen=True):
 
     _zoomer.setMagFactor(x, y)
 
-    # [[[TODO: JD - I think the screen is blanking out due to how gnome-mag
-    # does its thing.  See bug 503075. 
-    #
     if updateScreen:
         __updateROIDimensions()
         if (oldX > x) and (x < 1.5):
             _minROIX = _sourceDisplayBounds.x1
             _minROIY = _sourceDisplayBounds.y1
-            _maxROIX = _sourceDisplayBounds.x2
-            _maxROIY = _sourceDisplayBounds.y2
             __setROI(GNOME.Magnifier.RectBounds(_minROIX,
                                                 _minROIY,
-                                                _maxROIX,
-                                                _maxROIY))
+                                                _minROIX + _roiWidth,
+                                                _minROIY + _roiHeight))
         else:
             extents = orca_state.locusOfFocus.queryComponent().getExtents(0)
             __setROICenter(extents.x, extents.y)
@@ -1320,7 +1345,7 @@ def setZoomerColorFilter(colorFilter, updateScreen=True):
     - updateScreen:  Whether or not to update the screen
     """
 
-    if not _initialized:
+    if not _initialized or not isFilteringCapable():
         return
 
     if colorFilter == settings.MAG_COLOR_FILTERING_MODE_SATURATE_RED:
@@ -1352,6 +1377,18 @@ def setZoomerColorFilter(colorFilter, updateScreen=True):
     if updateScreen:
         _zoomer.markDirty(_roi)
 
+def isFilteringCapable():
+    """Returns True if we're able to take advantage of libcolorblind's color
+    filtering.
+    """
+
+    try:
+        capable = _magnifier.supportColorblindFilters()
+    except:
+        capable = False
+
+    return capable
+
 def updateMouseTracking(newMode):
     """Updates the mouse tracking mode.
 
@@ -1630,6 +1667,7 @@ def cycleZoomerType(script=None, inputEvent=None):
     # We don't want to stomp on any command-altered settings
     #
     toRestore = {}
+
     [levelX, levelY] = _zoomer.getMagFactor()
     if levelX != settings.magZoomFactor:
         toRestore['magZoomFactor'] = levelX
@@ -1647,11 +1685,8 @@ def cycleZoomerType(script=None, inputEvent=None):
              'magSmoothingMode': settings.MAG_SMOOTHING_MODE_BILINEAR,
              'magZoomerBorderColor': '#000000'})
 
-    setupMagnifier(orca_state.zoomerType)
+    setupMagnifier(orca_state.zoomerType, restore = toRestore)
 
-    # Now that we have our Magnifier set up, restore anything that needs
-    # restoring.
-    #
     if not orca_state.mouseEnhancementsEnabled:
         setMagnifierCrossHair(False)
         setMagnifierObjectColor("cursor-color",
@@ -1659,9 +1694,6 @@ def cycleZoomerType(script=None, inputEvent=None):
                                 False)
         setMagnifierCursor(False, False, 0)
 
-    if toRestore:
-        __setupZoomer(restore = toRestore)
-
     if orca_state.zoomerType == settings.MAG_ZOOMER_TYPE_FULL_SCREEN:
         if _fullScreenCapable:
             # Translators: magnification will use the full screen.
