@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2006 Sun Microsystems Inc.
+# Copyright 2006-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,15 +14,15 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Displays a GUI for the Orca Find window"""
 
-__id__        = "$Id: orca_gui_find.py 3077 2007-10-17 16:32:55Z richb $"
-__version__   = "$Revision: 3077 $"
-__date__      = "$Date: 2007-10-17 12:32:55 -0400 (Wed, 17 Oct 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: orca_gui_find.py 3882 2008-05-07 18:22:10Z richb $"
+__version__   = "$Revision: 3882 $"
+__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import os
@@ -40,9 +40,29 @@ from orca_i18n import _  # for gettext support
 
 OS = None
 
-class orcaFindGUI(orca_glade.GladeWrapper):
+class OrcaFindGUI(orca_glade.GladeWrapper):
 
-    def _init(self):
+    def __init__(self, fileName, windowName):
+        """Initialize the Orca configuration GUI.
+
+        Arguments:
+        - fileName: name of the Glade file.
+        - windowName: name of the component to get from the Glade file.
+        """
+
+        orca_glade.GladeWrapper.__init__(self, fileName, windowName)
+
+        # Initialize variables to None to keep pylint happy.
+        #
+        self.activeScript = None
+        self.caseSensitive = None
+        self.matchEntireWord = None
+        self.searchBackwards = None
+        self.searchString = None
+        self.startAtTop = None
+        self.windowWrap = None
+
+    def init(self):
         # Initialize the dialog box controls.
         self.searchString = ""
         self.searchBackwards = False
@@ -53,7 +73,7 @@ class orcaFindGUI(orca_glade.GladeWrapper):
 
         self.activeScript = orca_state.activeScript
 
-    def _showGUI(self):
+    def showGUI(self):
         """Show the Orca Find dialog. This assumes that the GUI has
         already been created.
         """
@@ -95,7 +115,7 @@ class orcaFindGUI(orca_glade.GladeWrapper):
         except:
             pass
 
-    def searchForEntryChanged(self,widget):
+    def searchForEntryChanged(self, widget):
         """Signal handler for the "changed" signal for the
            searchForEntry GtkEntry widget. The user has changed
            the string to be searched for.
@@ -228,10 +248,10 @@ def showFindUI():
                                  platform.package,
                                  "glade",
                                  "orca-find.glade")
-        OS = orcaFindGUI(gladeFile, "findDialog")
-        OS._init()
+        OS = OrcaFindGUI(gladeFile, "findDialog")
+        OS.init()
 
-    OS._showGUI()
+    OS.showGUI()
 
 def main():
     locale.setlocale(locale.LC_ALL, '')
