@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2007 Sun Microsystems Inc.
+# Copyright 2004-2008 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -14,21 +14,24 @@
 #
 # You should have received a copy of the GNU Library General Public
 # License along with this library; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
+# Boston MA  02110-1301 USA.
 
 """Manages the settings for Orca.  This will defer to user settings first, but
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 3963 2008-06-05 18:10:20Z wwalker $"
-__version__   = "$Revision: 3963 $"
-__date__      = "$Date: 2008-06-05 14:10:20 -0400 (Thu, 05 Jun 2008) $"
-__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
+__id__        = "$Id: settings.py 3912 2008-05-21 16:57:40Z richb $"
+__version__   = "$Revision: 3912 $"
+__date__      = "$Date: 2008-05-21 12:57:40 -0400 (Wed, 21 May 2008) $"
+__copyright__ = "Copyright (c) 2004-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import os
 import re
+import shutil
+
+import platform
 
 screenWidth = 640
 screenHeight = 480
@@ -105,6 +108,7 @@ userCustomizableSettings = [
     "brailleVerbosityLevel",
     "brailleRolenameStyle",
     "brailleSelectorIndicator",
+    "brailleLinkIndicator",
     "enableBrailleMonitor",
     "enableMagnifier",
     "enableMagLiveUpdating",
@@ -157,7 +161,9 @@ userCustomizableSettings = [
     "enableProgressBarUpdates",
     "progressBarUpdateInterval",
     "enableContractedBraille",
-    "brailleContractionTable"
+    "brailleContractionTable",
+    "enableMouseReview",
+    "mouseDwellDelay"
 ]
 
 # The name of the module that hold the user interface for the main window
@@ -221,12 +227,22 @@ brailleRolenameStyle    = BRAILLE_ROLENAME_STYLE_LONG
 # The values represent the character to be used in the attrOr
 # field of brlAPI's writeStruct.
 #
-BRAILLE_SEL_NONE = '\x00' # 00000000
-BRAILLE_SEL_7    = '\x40' # 01000000
-BRAILLE_SEL_8    = '\x80' # 10000000
-BRAILLE_SEL_BOTH = '\xc0' # 11000000
+BRAILLE_SEL_NONE = 0x00 # 00000000
+BRAILLE_SEL_7    = 0x40 # 01000000
+BRAILLE_SEL_8    = 0x80 # 10000000
+BRAILLE_SEL_BOTH = 0xc0 # 11000000
 brailleSelectorIndicator = BRAILLE_SEL_BOTH
 
+# Braille Link Indicator (see brailleLinkIndicator).
+# The values represent the character to be used in the attrOr
+# field of brlAPI's writeStruct.
+#
+BRAILLE_LINK_NONE = 0x00 # 00000000
+BRAILLE_LINK_7    = 0x40 # 01000000
+BRAILLE_LINK_8    = 0x80 # 10000000
+BRAILLE_LINK_BOTH = 0xc0 # 11000000
+brailleLinkIndicator = BRAILLE_LINK_BOTH
+
 # Speech punctuation levels (see verbalizePunctuationStyle).
 #
 PUNCTUATION_STYLE_NONE = 3
@@ -256,7 +272,7 @@ speechPitchDelta        = 0.5
 # The port to listen on if orca is to act as an HTTP server
 # (mainly as a speech server for self-voicing applications).
 #
-httpServerPort          = 20433
+httpServerPort          = 0
 
 # The number of attempts to retry setting up an HTTP server
 # connection (each time incrementing the port number by 1).
@@ -593,10 +609,10 @@ enabledBrailledTextAttributes = \
 # The values represent the character to be used in the attrOr
 # field of brlAPI's writeStruct.
 #
-TEXT_ATTR_BRAILLE_NONE = '\x00' # 00000000
-TEXT_ATTR_BRAILLE_7    = '\x40' # 01000000
-TEXT_ATTR_BRAILLE_8    = '\x80' # 10000000
-TEXT_ATTR_BRAILLE_BOTH = '\xc0' # 11000000
+TEXT_ATTR_BRAILLE_NONE = 0x00 # 00000000
+TEXT_ATTR_BRAILLE_7    = 0x40 # 01000000
+TEXT_ATTR_BRAILLE_8    = 0x80 # 10000000
+TEXT_ATTR_BRAILLE_BOTH = 0xc0 # 11000000
 textAttributesBrailleIndicator = TEXT_ATTR_BRAILLE_NONE
 
 # The limit to enable a repeat character count to be spoken.
@@ -681,14 +697,6 @@ useBonoboMain           = True
 #
 asyncMode               = True
 
-# A list of toolkits whose events we need to process synchronously.
-# The only one right now is the Java toolkit (see bug #531869), but
-# we put this here to allow more toolkits to be more easily added
-# and for Java to be removed if issues with asynchronous processing
-# of its events are ever resolved.
-#
-synchronousToolkits     = ["J2SE-access-bridge"]
-
 # If True, we output debug information for the event queue.  We
 # use this in addition to log level to prevent debug logic from
 # bogging down event handling.
@@ -733,6 +741,54 @@ def setAccessibilityEnabled(enable):
     except:
         return False
 
+# Obtain/set information regarding whether Orca is autostarted for this 
+# user at login time.
+#
+def isOrcaAutostarted():
+    """Return an indication of whether Orca autostart at login time is enabled.
+    """
+
+    userDesktopFile = os.path.join(os.environ["HOME"], ".config", \
+                                   "autostart", "orca.desktop")
+    return os.path.exists(userDesktopFile)
+
+def setOrcaAutostart(enable):
+    """Enable or disable the autostart of Orca at login time.
+
+    Arguments:
+    - enable: if True, whether Orca autostart at login time is enabled.
+
+    Returns an indication of whether the operation was successful.
+    """
+
+    status = True
+    systemDesktopFile = os.path.join(platform.prefix, platform.datadirname, \
+                                     "applications", "orca.desktop")
+    autostartDir = os.path.join(os.environ["HOME"], ".config", "autostart")
+    userDesktopFile = os.path.join(os.environ["HOME"], ".config", \
+                                   "autostart", "orca.desktop")
+
+    try:
+        # If the user wants to enable Orca autostart, then copy the
+        # orca.desktop file from /usr/share/applications to
+        # ~/.config/autostart directory (creating the autostart directory
+        # if neccessary).
+        #
+        # If the user wants to disable this feature, then just remove 
+        # ~/.config/autostart/orca.desktop (if it exists).
+        #
+        if enable:
+            try:
+                os.chdir(autostartDir)
+            except:
+                os.mkdir(autostartDir)
+            shutil.copy(systemDesktopFile, userDesktopFile)
+        else:
+            os.remove(userDesktopFile)
+    except:
+        status = False
+    return status
+
 # Obtain/set information regarding whether the gksu keyboard grab is enabled
 # or not.
 #
@@ -773,7 +829,8 @@ settingsPackages          = ["app-settings"]
 # REMEMBER: to make something a package, the directory has to
 # have a __init__.py file in it.
 #
-scriptPackages          = ["orca-scripts", "scripts"]
+scriptPackages          = ["orca-scripts", "scripts", 
+                           "scripts.apps", "scripts.toolkits"]
 
 # A list that helps us map application names to script module
 # names.  The key is the name of an application, and the value is
@@ -831,22 +888,22 @@ def getScriptModuleName(app):
 # I wish the AT-SPI spec'd this out as machine readable (unlocalized)
 # names, but it's what we're stuck with (unfortunately).
 #
-setScriptMapping(re.compile(_('[\S\s]*StarOffice[\s\S]*')), "StarOffice")
+setScriptMapping(re.compile(_('[\S\s]*StarOffice[\s\S]*')), "soffice")
 
 # Translators: see the regular expression note above.  This is for
 # OpenOffice and StarOffice.
 #
-setScriptMapping(re.compile(_('soffice.bin')), "StarOffice")
+setScriptMapping(re.compile(_('soffice.bin')), "soffice")
 
 # Translators: see the regular expression note above.  This is for
 # OpenOffice and StarOffice.
 #
-setScriptMapping(re.compile(_('soffice')), "StarOffice")
+setScriptMapping(re.compile(_('soffice')), "soffice")
 
 # Translators: see the regular expression note above.  This is for the
 # Evolution mail application.
 #
-setScriptMapping(re.compile(_('[Ee]volution')), "Evolution")
+setScriptMapping(re.compile(_('[Ee]volution')), "evolution")
 
 # Translators: see the regular expression note above.  This is for the
 # help application (yelp).
@@ -887,9 +944,9 @@ setScriptMapping(re.compile(_('bug-buddy')), "gnome_segv2")
 setScriptMapping(re.compile(_('vte')), "gnome-terminal")
 
 # Translators: see the regular expression note above.  This is for
-# supporting pidgin, which is the new name for gaim.
+# supporting gaim, which has recently be renamed to pidgin.
 #
-setScriptMapping(re.compile(_('pidgin')), "gaim")
+setScriptMapping(re.compile(_('gaim')), "pidgin")
 
 # Show deprecated messeges in debug output.
 # Set this to True to help find potential pyatspi porting problems
@@ -927,3 +984,17 @@ brailleContractionTable = ''
 # Use Collection Interface?
 # 
 useCollection = True
+
+
+# Report object under mouse.
+#
+enableMouseReview = False
+
+# Mouse dwell delay in milliseconds for mouse review mode. 
+# If the value is zero, the review will be read time.
+#
+mouseDwellDelay = 0
+
+# Maximum allowed drift while pointer is dwelling in mouse review mode.
+# 
+mouseDwellMaxDrift = 3
